@@ -1,24 +1,24 @@
 /* Generic symbol-table support for the BFD library.
    Copyright 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002
+   2000, 2001, 2002, 2003
    Free Software Foundation, Inc.
    Written by Cygnus Support.
 
-This file is part of BFD, the Binary File Descriptor library.
+   This file is part of BFD, the Binary File Descriptor library.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /*
 SECTION
@@ -74,10 +74,10 @@ SUBSECTION
 |         if (storage_needed < 0)
 |           FAIL
 |
-|	  if (storage_needed == 0) {
-|	     return ;
-|	  }
-|	  symbol_table = (asymbol **) xmalloc (storage_needed);
+|	  if (storage_needed == 0)
+|	    return;
+|	  
+|	  symbol_table = xmalloc (storage_needed);
 |	    ...
 |	  number_of_symbols =
 |	     bfd_canonicalize_symtab (abfd, symbol_table);
@@ -85,9 +85,8 @@ SUBSECTION
 |         if (number_of_symbols < 0)
 |           FAIL
 |
-|	  for (i = 0; i < number_of_symbols; i++) {
-|	     process_symbol (symbol_table[i]);
-|	  }
+|	  for (i = 0; i < number_of_symbols; i++)
+|	    process_symbol (symbol_table[i]);
 
 	All storage for the symbols themselves is in an objalloc
 	connected to the BFD; it is freed when the BFD is closed.
@@ -108,25 +107,26 @@ SUBSECTION
 	example showing the creation of a symbol table with only one element:
 
 |	#include "bfd.h"
-|	main()
+|	int main (void)
 |	{
 |	  bfd *abfd;
 |	  asymbol *ptrs[2];
 |	  asymbol *new;
 |
-|	  abfd = bfd_openw("foo","a.out-sunos-big");
-|	  bfd_set_format(abfd, bfd_object);
-|	  new = bfd_make_empty_symbol(abfd);
+|	  abfd = bfd_openw ("foo","a.out-sunos-big");
+|	  bfd_set_format (abfd, bfd_object);
+|	  new = bfd_make_empty_symbol (abfd);
 |	  new->name = "dummy_symbol";
-|	  new->section = bfd_make_section_old_way(abfd, ".text");
+|	  new->section = bfd_make_section_old_way (abfd, ".text");
 |	  new->flags = BSF_GLOBAL;
 |	  new->value = 0x12345;
 |
 |	  ptrs[0] = new;
-|	  ptrs[1] = (asymbol *)0;
+|	  ptrs[1] = 0;
 |
-|	  bfd_set_symtab(abfd, ptrs, 1);
-|	  bfd_close(abfd);
+|	  bfd_set_symtab (abfd, ptrs, 1);
+|	  bfd_close (abfd);
+|	  return 0;
 |	}
 |
 |	./makesym
@@ -192,7 +192,7 @@ CODE_FRAGMENT
 .     instead, except that some symbols point to the global sections
 .     bfd_{abs,com,und}_section.  This could be fixed by making
 .     these globals be per-bfd (or per-target-flavor).  FIXME.  *}
-.  struct _bfd *the_bfd; {* Use bfd_asymbol_bfd(sym) to access this field.  *}
+.  struct bfd *the_bfd; {* Use bfd_asymbol_bfd(sym) to access this field.  *}
 .
 .  {* The text of the symbol. The name is left alone, and not copied; the
 .     application may not alter it.  *}
@@ -293,12 +293,12 @@ CODE_FRAGMENT
 .  {* A pointer to the section to which this symbol is
 .     relative.  This will always be non NULL, there are special
 .     sections for undefined and absolute symbols.  *}
-.  struct sec *section;
+.  struct bfd_section *section;
 .
 .  {* Back end special data.  *}
 .  union
 .    {
-.      PTR p;
+.      void *p;
 .      bfd_vma i;
 .    }
 .  udata;
@@ -314,10 +314,6 @@ CODE_FRAGMENT
 #include "bfdlink.h"
 #include "aout/stab_gnu.h"
 
-static char coff_section_type PARAMS ((const char *));
-static char decode_section_type PARAMS ((const struct sec *));
-static int cmpindexentry PARAMS ((const PTR, const PTR));
-
 /*
 DOCDD
 INODE
@@ -346,7 +342,7 @@ FUNCTION
 	bfd_is_local_label
 
 SYNOPSIS
-        bfd_boolean bfd_is_local_label(bfd *abfd, asymbol *sym);
+        bfd_boolean bfd_is_local_label (bfd *abfd, asymbol *sym);
 
 DESCRIPTION
 	Return TRUE if the given symbol @var{sym} in the BFD @var{abfd} is
@@ -354,9 +350,7 @@ DESCRIPTION
 */
 
 bfd_boolean
-bfd_is_local_label (abfd, sym)
-     bfd *abfd;
-     asymbol *sym;
+bfd_is_local_label (bfd *abfd, asymbol *sym)
 {
   /* The BSF_SECTION_SYM check is needed for IA-64, where every label that
      starts with '.' is local.  This would accidentally catch section names
@@ -382,7 +376,7 @@ DESCRIPTION
 	local label.
 
 .#define bfd_is_local_label_name(abfd, name) \
-.     BFD_SEND (abfd, _bfd_is_local_label_name, (abfd, name))
+.  BFD_SEND (abfd, _bfd_is_local_label_name, (abfd, name))
 .
 */
 
@@ -398,8 +392,7 @@ DESCRIPTION
 	including the NULL.
 
 .#define bfd_canonicalize_symtab(abfd, location) \
-.     BFD_SEND (abfd, _bfd_canonicalize_symtab,\
-.                  (abfd, location))
+.  BFD_SEND (abfd, _bfd_canonicalize_symtab, (abfd, location))
 .
 */
 
@@ -408,7 +401,8 @@ FUNCTION
 	bfd_set_symtab
 
 SYNOPSIS
-	bfd_boolean bfd_set_symtab (bfd *abfd, asymbol **location, unsigned int count);
+	bfd_boolean bfd_set_symtab
+	  (bfd *abfd, asymbol **location, unsigned int count);
 
 DESCRIPTION
 	Arrange that when the output BFD @var{abfd} is closed,
@@ -417,12 +411,9 @@ DESCRIPTION
 */
 
 bfd_boolean
-bfd_set_symtab (abfd, location, symcount)
-     bfd *abfd;
-     asymbol **location;
-     unsigned int symcount;
+bfd_set_symtab (bfd *abfd, asymbol **location, unsigned int symcount)
 {
-  if ((abfd->format != bfd_object) || (bfd_read_p (abfd)))
+  if (abfd->format != bfd_object || bfd_read_p (abfd))
     {
       bfd_set_error (bfd_error_invalid_operation);
       return FALSE;
@@ -438,29 +429,23 @@ FUNCTION
 	bfd_print_symbol_vandf
 
 SYNOPSIS
-	void bfd_print_symbol_vandf(bfd *abfd, PTR file, asymbol *symbol);
+	void bfd_print_symbol_vandf (bfd *abfd, void *file, asymbol *symbol);
 
 DESCRIPTION
 	Print the value and flags of the @var{symbol} supplied to the
 	stream @var{file}.
 */
 void
-bfd_print_symbol_vandf (abfd, arg, symbol)
-     bfd *abfd;
-     PTR arg;
-     asymbol *symbol;
+bfd_print_symbol_vandf (bfd *abfd, void *arg, asymbol *symbol)
 {
-  FILE *file = (FILE *) arg;
+  FILE *file = arg;
+
   flagword type = symbol->flags;
-  if (symbol->section != (asection *) NULL)
-    {
-      bfd_fprintf_vma (abfd, file,
-		       symbol->value + symbol->section->vma);
-    }
+
+  if (symbol->section != NULL)
+    bfd_fprintf_vma (abfd, file, symbol->value + symbol->section->vma);
   else
-    {
-      bfd_fprintf_vma (abfd, file, symbol->value);
-    }
+    bfd_fprintf_vma (abfd, file, symbol->value);
 
   /* This presumes that a symbol can not be both BSF_DEBUGGING and
      BSF_DYNAMIC, nor more than one of BSF_FUNCTION, BSF_FILE, and
@@ -495,7 +480,7 @@ DESCRIPTION
 	information, and will cause problems later on.
 
 .#define bfd_make_empty_symbol(abfd) \
-.     BFD_SEND (abfd, _bfd_make_empty_symbol, (abfd))
+.  BFD_SEND (abfd, _bfd_make_empty_symbol, (abfd))
 .
 */
 
@@ -514,11 +499,10 @@ DESCRIPTION
 */
 
 asymbol *
-_bfd_generic_make_empty_symbol (abfd)
-     bfd *abfd;
+_bfd_generic_make_empty_symbol (bfd *abfd)
 {
   bfd_size_type amt = sizeof (asymbol);
-  asymbol *new = (asymbol *) bfd_zalloc (abfd, amt);
+  asymbol *new = bfd_zalloc (abfd, amt);
   if (new)
     new->the_bfd = abfd;
   return new;
@@ -534,7 +518,7 @@ DESCRIPTION
 	yet to be worked out.
 
 .#define bfd_make_debug_symbol(abfd,ptr,size) \
-.        BFD_SEND (abfd, _bfd_make_debug_symbol, (abfd, ptr, size))
+.  BFD_SEND (abfd, _bfd_make_debug_symbol, (abfd, ptr, size))
 .
 */
 
@@ -578,8 +562,7 @@ static const struct section_to_type stt[] =
    't' as well as .text */
 
 static char
-coff_section_type (s)
-     const char *s;
+coff_section_type (const char *s)
 {
   const struct section_to_type *t;
 
@@ -598,8 +581,7 @@ coff_section_type (s)
    we could perhaps obsolete coff_section_type.  */
 
 static char
-decode_section_type (section)
-     const struct sec *section;
+decode_section_type (const struct bfd_section *section)
 {
   if (section->flags & SEC_CODE)
     return 't';
@@ -621,6 +603,8 @@ decode_section_type (section)
     }
   if (section->flags & SEC_DEBUGGING)
     return 'N';
+  if ((section->flags & SEC_HAS_CONTENTS) && (section->flags & SEC_READONLY))
+    return 'n';
 
   return '?';
 }
@@ -634,11 +618,10 @@ DESCRIPTION
 	class of @var{symbol}, or '?' for an unknown class.
 
 SYNOPSIS
-	int bfd_decode_symclass(asymbol *symbol);
+	int bfd_decode_symclass (asymbol *symbol);
 */
 int
-bfd_decode_symclass (symbol)
-     asymbol *symbol;
+bfd_decode_symclass (asymbol *symbol)
 {
   char c;
 
@@ -710,8 +693,7 @@ SYNOPSIS
 */
 
 bfd_boolean
-bfd_is_undefined_symclass (symclass)
-     int symclass;
+bfd_is_undefined_symclass (int symclass)
 {
   return symclass == 'U' || symclass == 'w' || symclass == 'v';
 }
@@ -726,13 +708,11 @@ DESCRIPTION
 	calling this function.
 
 SYNOPSIS
-	void bfd_symbol_info(asymbol *symbol, symbol_info *ret);
+	void bfd_symbol_info (asymbol *symbol, symbol_info *ret);
 */
 
 void
-bfd_symbol_info (symbol, ret)
-     asymbol *symbol;
-     symbol_info *ret;
+bfd_symbol_info (asymbol *symbol, symbol_info *ret)
 {
   ret->type = bfd_decode_symclass (symbol);
 
@@ -749,7 +729,8 @@ FUNCTION
 	bfd_copy_private_symbol_data
 
 SYNOPSIS
-	bfd_boolean bfd_copy_private_symbol_data(bfd *ibfd, asymbol *isym, bfd *obfd, asymbol *osym);
+	bfd_boolean bfd_copy_private_symbol_data
+	  (bfd *ibfd, asymbol *isym, bfd *obfd, asymbol *osym);
 
 DESCRIPTION
 	Copy private symbol information from @var{isym} in the BFD
@@ -761,8 +742,8 @@ DESCRIPTION
 	Not enough memory exists to create private data for @var{osec}.
 
 .#define bfd_copy_private_symbol_data(ibfd, isymbol, obfd, osymbol) \
-.     BFD_SEND (obfd, _bfd_copy_private_symbol_data, \
-.		(ibfd, isymbol, obfd, osymbol))
+.  BFD_SEND (obfd, _bfd_copy_private_symbol_data, \
+.            (ibfd, isymbol, obfd, osymbol))
 .
 */
 
@@ -771,11 +752,10 @@ DESCRIPTION
    version.  It just uses BFD asymbol structures as mini symbols.  */
 
 long
-_bfd_generic_read_minisymbols (abfd, dynamic, minisymsp, sizep)
-     bfd *abfd;
-     bfd_boolean dynamic;
-     PTR *minisymsp;
-     unsigned int *sizep;
+_bfd_generic_read_minisymbols (bfd *abfd,
+			       bfd_boolean dynamic,
+			       void **minisymsp,
+			       unsigned int *sizep)
 {
   long storage;
   asymbol **syms = NULL;
@@ -790,7 +770,7 @@ _bfd_generic_read_minisymbols (abfd, dynamic, minisymsp, sizep)
   if (storage == 0)
     return 0;
 
-  syms = (asymbol **) bfd_malloc ((bfd_size_type) storage);
+  syms = bfd_malloc (storage);
   if (syms == NULL)
     goto error_return;
 
@@ -801,7 +781,7 @@ _bfd_generic_read_minisymbols (abfd, dynamic, minisymsp, sizep)
   if (symcount < 0)
     goto error_return;
 
-  *minisymsp = (PTR) syms;
+  *minisymsp = syms;
   *sizep = sizeof (asymbol *);
   return symcount;
 
@@ -816,13 +796,11 @@ _bfd_generic_read_minisymbols (abfd, dynamic, minisymsp, sizep)
    an asymbol.  We don't worry about the sym argument we are passed;
    we just return the asymbol the minisymbol points to.  */
 
-/*ARGSUSED*/
 asymbol *
-_bfd_generic_minisymbol_to_symbol (abfd, dynamic, minisym, sym)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     bfd_boolean dynamic ATTRIBUTE_UNUSED;
-     const PTR minisym;
-     asymbol *sym ATTRIBUTE_UNUSED;
+_bfd_generic_minisymbol_to_symbol (bfd *abfd ATTRIBUTE_UNUSED,
+				   bfd_boolean dynamic ATTRIBUTE_UNUSED,
+				   const void *minisym,
+				   asymbol *sym ATTRIBUTE_UNUSED)
 {
   return *(asymbol **) minisym;
 }
@@ -857,12 +835,10 @@ struct indexentry
 /* Compare two indexentry structures.  This is called via qsort.  */
 
 static int
-cmpindexentry (a, b)
-     const PTR a;
-     const PTR b;
+cmpindexentry (const void *a, const void *b)
 {
-  const struct indexentry *contestantA = (const struct indexentry *) a;
-  const struct indexentry *contestantB = (const struct indexentry *) b;
+  const struct indexentry *contestantA = a;
+  const struct indexentry *contestantB = b;
 
   if (contestantA->val < contestantB->val)
     return -1;
@@ -903,17 +879,15 @@ struct stab_find_info
 };
 
 bfd_boolean
-_bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
-				     pfilename, pfnname, pline, pinfo)
-     bfd *abfd;
-     asymbol **symbols;
-     asection *section;
-     bfd_vma offset;
-     bfd_boolean *pfound;
-     const char **pfilename;
-     const char **pfnname;
-     unsigned int *pline;
-     PTR *pinfo;
+_bfd_stab_section_find_nearest_line (bfd *abfd,
+				     asymbol **symbols,
+				     asection *section,
+				     bfd_vma offset,
+				     bfd_boolean *pfound,
+				     const char **pfilename,
+				     const char **pfnname,
+				     unsigned int *pline,
+				     void **pinfo)
 {
   struct stab_find_info *info;
   bfd_size_type stabsize, strsize;
@@ -951,7 +925,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 #define VALOFF (8)
 #define STABSIZE (12)
 
-  info = (struct stab_find_info *) *pinfo;
+  info = *pinfo;
   if (info != NULL)
     {
       if (info->stabsec == NULL || info->strsec == NULL)
@@ -972,7 +946,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       char *function_name;
       bfd_size_type amt = sizeof *info;
 
-      info = (struct stab_find_info *) bfd_zalloc (abfd, amt);
+      info = bfd_zalloc (abfd, amt);
       if (info == NULL)
 	return FALSE;
 
@@ -987,15 +961,15 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 	{
 	  /* No stabs debugging information.  Set *pinfo so that we
              can return quickly in the info != NULL case above.  */
-	  *pinfo = (PTR) info;
+	  *pinfo = info;
 	  return TRUE;
 	}
 
       stabsize = info->stabsec->_raw_size;
       strsize = info->strsec->_raw_size;
 
-      info->stabs = (bfd_byte *) bfd_alloc (abfd, stabsize);
-      info->strs = (bfd_byte *) bfd_alloc (abfd, strsize);
+      info->stabs = bfd_alloc (abfd, stabsize);
+      info->strs = bfd_alloc (abfd, strsize);
       if (info->stabs == NULL || info->strs == NULL)
 	return FALSE;
 
@@ -1005,14 +979,14 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 					 (bfd_vma) 0, strsize))
 	return FALSE;
 
-      /* If this is a relocateable object file, we have to relocate
+      /* If this is a relocatable object file, we have to relocate
 	 the entries in .stab.  This should always be simple 32 bit
 	 relocations against symbols defined in this object file, so
 	 this should be no big deal.  */
       reloc_size = bfd_get_reloc_upper_bound (abfd, info->stabsec);
       if (reloc_size < 0)
 	return FALSE;
-      reloc_vector = (arelent **) bfd_malloc ((bfd_size_type) reloc_size);
+      reloc_vector = bfd_malloc (reloc_size);
       if (reloc_vector == NULL && reloc_size != 0)
 	return FALSE;
       reloc_count = bfd_canonicalize_reloc (abfd, info->stabsec, reloc_vector,
@@ -1105,7 +1079,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 
       amt = info->indextablesize;
       amt *= sizeof (struct indexentry);
-      info->indextable = (struct indexentry *) bfd_alloc (abfd, amt);
+      info->indextable = bfd_alloc (abfd, amt);
       if (info->indextable == NULL)
 	return FALSE;
 
@@ -1226,7 +1200,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       qsort (info->indextable, (size_t) i, sizeof (struct indexentry),
 	     cmpindexentry);
 
-      *pinfo = (PTR) info;
+      *pinfo = info;
     }
 
   /* We are passed a section relative offset.  The offsets in the
@@ -1245,12 +1219,11 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
   else
 #endif
     {
-      /* Cache non-existant or invalid.  Do binary search on
-         indextable.  */
-
       long low, high;
       long mid = -1;
 
+      /* Cache non-existant or invalid.  Do binary search on
+         indextable.  */
       indexentry = NULL;
 
       low = 0;
@@ -1305,9 +1278,11 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 	case N_SLINE:
 	case N_DSLINE:
 	case N_BSLINE:
-	  /* A line number.  The value is relative to the start of the
-             current function.  */
-	  val = indexentry->val + bfd_get_32 (abfd, stab + VALOFF);
+	  /* A line number.  If the function was specified, then the value
+	     is relative to the start of the function.  Otherwise, the
+	     value is an absolute address.  */
+	  val = ((indexentry->function_name ? indexentry->val : 0)
+		 + bfd_get_32 (abfd, stab + VALOFF));
 	  /* If this line starts before our desired offset, or if it's
 	     the first line we've been able to find, use it.  The
 	     !saw_line check works around a bug in GCC 2.95.3, which emits
@@ -1359,7 +1334,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 	  if (info->filename != NULL)
 	    free (info->filename);
 	  len = strlen (file_name) + 1;
-	  info->filename = (char *) bfd_malloc ((bfd_size_type) dirlen + len);
+	  info->filename = bfd_malloc (dirlen + len);
 	  if (info->filename == NULL)
 	    return FALSE;
 	  memcpy (info->filename, directory_name, dirlen);
@@ -1376,7 +1351,6 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       /* This will typically be something like main:F(0,1), so we want
          to clobber the colon.  It's OK to change the name, since the
          string is in our own local storage anyhow.  */
-
       s = strchr (indexentry->function_name, ':');
       if (s != NULL)
 	*s = '\0';
